"""基础设施层 — 日志、配置、异常体系、文件工具。"""

from .config import (
    AccountConfig,
    BattleConfig,
    ConfigManager,
    DailyAutomationConfig,
    DecisiveConfig,
    EmulatorConfig,
    ExerciseConfig,
    FightConfig,
    LogConfig,
    NodeConfig,
    OCRConfig,
    UserConfig,
)
from .exceptions import (
    ActionFailedError,
    AutoWSGRError,
    ConfigError,
    CriticalError,
    DockFullError,
    EmulatorConnectionError,
    EmulatorError,
    EmulatorNotFoundError,
    GameError,
    OCRError,
    PageNotFoundError,
    ResourceError,
    UIError,
    VisionError,
)
from .file_utils import load_yaml, merge_dicts, resolve_plan_path, save_yaml
from .logger import caller_info, get_logger, save_image, setup_logger


__all__ = [
    'AccountConfig',
    'ActionFailedError',
    'AutoWSGRError',
    'BattleConfig',
    'ConfigError',
    'ConfigManager',
    'CriticalError',
    'DailyAutomationConfig',
    'DecisiveConfig',
    'DockFullError',
    'EmulatorConfig',
    'EmulatorConnectionError',
    'EmulatorError',
    'EmulatorNotFoundError',
    'ExerciseConfig',
    'FightConfig',
    'GameError',
    'LogConfig',
    'NodeConfig',
    'OCRConfig',
    'OCRError',
    'PageNotFoundError',
    'ResourceError',
    'UIError',
    'UserConfig',
    'VisionError',
    'caller_info',
    'get_logger',
    'load_yaml',
    'merge_dicts',
    'resolve_plan_path',
    'save_image',
    'save_yaml',
    'setup_logger',
]
