"""面板 Mixin 子包。"""

from autowsgr.ui.map.panels.campaign import CampaignPanelMixin
from autowsgr.ui.map.panels.decisive import DecisivePanelMixin
from autowsgr.ui.map.panels.exercise import ExercisePanelMixin
from autowsgr.ui.map.panels.expedition import ExpeditionPanelMixin
from autowsgr.ui.map.panels.sortie import (
    LootShipCount,
    SortiePanelMixin,
    recognize_loot_count,
    recognize_ship_count,
)


__all__ = [
    'CampaignPanelMixin',
    'DecisivePanelMixin',
    'ExercisePanelMixin',
    'ExpeditionPanelMixin',
    'LootShipCount',
    'SortiePanelMixin',
    'recognize_loot_count',
    'recognize_ship_count',
]
