"""视觉层 — 像素特征检测 + 模板图像匹配 + OCR。

公开 API::

    # 像素检测
    from autowsgr.vision import (
        Color,
        PixelRule,
        PixelSignature,
        MatchStrategy,
        PixelChecker,
        PixelMatchResult,
        PixelDetail,
    )

    # 图像模板匹配
    from autowsgr.vision import (
        ROI,
        ImageTemplate,
        ImageRule,
        ImageSignature,
        ImageChecker,
        ImageMatchResult,
        ImageMatchDetail,
    )

    # OCR
    from autowsgr.vision import OCREngine, OCRResult
"""

from .api_dll import ApiDll, get_api_dll
from .image_matcher import TEMPLATE_SOURCE_RESOLUTION, ImageChecker
from .image_template import (
    ImageMatchDetail,
    ImageMatchResult,
    ImageRule,
    ImageSignature,
    ImageTemplate,
)
from .matcher import PixelChecker
from .ocr import EasyOCREngine, OCREngine, OCRResult, ShipNameMismatchError, apply_ship_patches
from .pixel import (
    Color,
    CompositePixelSignature,
    MatchStrategy,
    PixelDetail,
    PixelMatchResult,
    PixelRule,
    PixelSignature,
)
from .roi import ROI


__all__ = [
    # matcher (pixel)
    'Color',
    'CompositePixelSignature',
    'MatchStrategy',
    'PixelChecker',
    'PixelDetail',
    'PixelMatchResult',
    'PixelRule',
    'PixelSignature',
    # image_matcher (template)
    'ROI',
    'ImageTemplate',
    'ImageRule',
    'ImageSignature',
    'ImageChecker',
    'ImageMatchResult',
    'ImageMatchDetail',
    'TEMPLATE_SOURCE_RESOLUTION',
    # ocr
    'OCREngine',
    'OCRResult',
    'EasyOCREngine',
    'ShipNameMismatchError',
    'apply_ship_patches',
    # api_dll
    'ApiDll',
    'get_api_dll',
]
